/************************************************************************
*	Spectrally based Technicolor Emulation Version 1.0		*
*									*
*	Thomas Edwards							*
*									*
*	NCCA Bournemouth 2006						*
*									*
*	All queries to : 						*
*									*
*		frodules@yahoo.co.uk					*
*									*
/***********************************************************************/

#include <iostream>
#include <string>

using namespace std;

#ifdef __cplusplus
extern "C" {
#endif

#include "lib/NCCAPixmap.h"
#include "lib/PixFileIO.h"

#ifdef __cplusplus
}
#endif

#include "point.h"
#include "curve.h"
#include "pixels.h"
#include "matrix.h"
#include "spectral.h"


//globals to store image width and height
int img_w = 1;
int img_h = 1;

/*matrix for converting between RGB and XYZ colour spaces*/
matrix rgb_xyz(	0.412424,	0.212656,	0.0193324,	0.0,
		0.357579,	0.715158,	0.119193,	0.0,
		0.180464,	0.0721856,	0.950444,	0.0,
		0.0,		0.0,		0.0,		1.0);

float limits[6] = {1,0,1,0,1,0};

void init(curvy& wr, curvy& film, curvy& dye, curvy& hd, curvy& xyz, curvy& ss)
{
	wr.read("curves/wratten/green61.cv" , "curves/wratten/magenta32.cv" , "curves/wratten/yellow12.cv");
	film.read("curves/kodak2393/red_bw.jpg.cv","curves/kodak2393/green_bw.jpg.cv","curves/kodak2393/blue_bw.jpg.cv");
	dye.read("curves/dyes/cyan.jpg.cv","curves/dyes/mag.jpg.cv","curves/dyes/yel.jpg.cv");
	hd.read("curves/filmss/hd.jpg.cv","curves/filmss/hd.jpg.cv","curves/filmss/hd.jpg.cv");
	xyz.read("curves/xyz_functions/xbar.match","curves/xyz_functions/ybar.match","curves/xyz_functions/zbar.match");
	ss.read("curves/filmss/doubleX.jpg.cv","curves/filmss/doubleX.jpg.cv","curves/filmss/ortho_ilford.jpg.cv");
}

int main(int argc, char* argv[])
{
	if(argc != 3)
	{
		cerr<<"error : incorrect number of arguments\n\nmain image_in image_out\n";
		exit(1);
	}
	
cout<<"takes around two minutes for a PAL resolution image"<<endl;

NCCAPixmap imgin,imgout;
curvy wr,film,dye,hd,xyz,ss;
matrix film_mat;
init(wr,film,dye,hd,xyz,ss);

/*create rough colour transformation matrix based on film characteristics*/
fill_film_col_matrix(film,film_mat);
load_img(imgin,argv[1]);

img_w = imgin.width;
img_h = imgin.height;

imgout=newPixmapLike(imgin);
copy_pixels(imgin,imgout);

curvy** specimg = spec_pixels(imgin.width,imgin.height);

matrix_transform(imgout,film_mat);

/*convert approximate spectral data from pixel data*/
xyz_spectral(imgout, xyz, specimg);

/*simulate coloured filters*/
filter(specimg, wr);

/*simulate film spectral sensitivity*/
filmss(specimg, ss, imgout);

//dye_filter(imgout, dye, specimg);

/*convert spectral data back to pixel values*/
spectral_xyz(specimg, xyz, imgout);

savePixmap(imgout,argv[2]);
	
if(specimg != NULL)
	delete [] specimg;
	
return 0;
}



