#include <iostream>
#include <math.h>

#ifdef __cplusplus
extern "C" {
#endif

#include "lib/NCCAPixmap.h"
#include "lib/PixFileIO.h"

#ifdef __cplusplus
}
#endif

#include "pixels.h"

extern float limits[6];
extern int img_w;
extern int img_h;

void load_img(NCCAPixmap& img,char* filename)
{
	img=loadPixmap(filename);
	if(img.data==NULL)
	{
		fprintf(stderr,"Couldn't load %s\n",filename);
		exit(-1);
	}
}

/*
**void minmax_rgb(NCCAPixel p, float* lim)
**find min and max pixel channel values in the whole image
**and store in global array lim[6]
*/
void minmax_rgb(NCCAPixel p, float* lim)
{
	if(p.r < lim[0])lim[0] = p.r;
	if(p.g < lim[2])lim[2] = p.g;
	if(p.b < lim[4])lim[4] = p.b;
	
	if(p.r > lim[1])lim[1] = p.r;
	if(p.g > lim[3])lim[3] = p.g;
	if(p.b > lim[5])lim[5] = p.b;
}

/*
**void fit_range(NCCAPixmap& img)
**fit transformed image to original range
*/
void fit_range(NCCAPixmap& img)
{
float limited[6] = {1,0,1,0,1,0};
float scale[3] = {0,0,0};
NCCAPixel p;
int i=0,j=0;

	for(i=0; i<img_w; i++)
	{
		for(j=0; j<img_h; j++)
		{
		p=getPixel(img,i,j);
		minmax_rgb(p,limited);
		}
	}
	
	for(i = 0; i < 3; ++i)
		scale[i] = (limited[2*i+1]-limited[2*i]) / (limits[2*i+1]-limits[2*i]);
	
	for(i=0; i<img_w; i++)
	{
		for(j=0; j<img_h; j++)
		{
		p=getPixel(img,i,j);
		p.r += p.r * scale[0] + limits[0];
		p.g += p.g * scale[1] + limits[2];
		p.b += p.b * scale[2] + limits[4];
		setPixelColour(img,i,j,p);
		}
	}
}

/*
**void copy_pixels(NCCAPixmap& oldi, NCCAPixmap& newi)
**duplicate pixels between identical Pixmaps
*/
void copy_pixels(NCCAPixmap& oldi, NCCAPixmap& newi)
{
NCCAPixel p;

	for(int i=0; i<oldi.width; i++)
	{
		for(int j=0; j<oldi.height; j++)
		{
		p=getPixel(oldi,i,j);
		minmax_rgb(p,limits);
		setPixelColour(newi,i,j,p);
		}
	}
}

/*
**curvy** spec_pixels(int w,int h)
**initialise array of curves to store spectral data for each pixel
*/
curvy** spec_pixels(int w,int h)
{
curvy** temp;
temp = new curvy*[w];
	for(int i = 0; i < w; ++i)
	{
		temp[i] = new curvy[h];
	}
return temp;
}

/*general utility to print formatted pixel values*/
void print_pixel(NCCAPixel& p)
{
cout<<p.r<<"\t"<<p.g<<"\t"<<p.b<<"\t"<<endl<<endl;
}

/*
**void three_strip_cheap(NCCCAPixmap *img)
**pixel based approximation of the three strip process
*/
void three_strip_cheap(NCCAPixmap& img)
{
NCCAPixel p;
float r,g,b;
float rmat,gmat,bmat;
	for(int i=0; i<img_w; i++)
	{
		for(int j=0;j<img_h; j++)
		{
		p=getPixel(img,i,j);
		rmat=1-(p.r*2-(p.g+p.b));
		gmat=1-(p.g*2-(p.r+p.b));
		bmat=1-(p.b*2-(p.g+p.r));
		r=p.r*gmat*bmat;
		g=p.g*bmat*rmat;
		b=p.b*rmat*gmat;
		p.r=r;
		p.g=g;
		p.b=b;
		setPixelColour(img,i,j,p);
		}
	}

}

/*
**void pix_mat(NCCAPixel& p, matrix mat)
**transform pixel by the given 4*4 matrix
*/
void pix_mat(NCCAPixel& p, matrix mat)
{
double r=0.0,g=0.0,b=0.0;

	r = p.r * mat.get(0,0) + p.g * mat.get(0,1) + p.b * mat.get(0,2) + mat.get(0,3);
	g = p.r * mat.get(1,0) + p.g * mat.get(1,1) + p.b * mat.get(1,2) + mat.get(1,3);
	b = p.r * mat.get(2,0) + p.g * mat.get(2,1) + p.b * mat.get(2,2) + mat.get(2,3);
	
	p.r = r;
	p.g = g;
	p.b = b;
}

/*
**void matrix_transform(NCCAPixmap& img, matrix mat)
**transform image by the given 4*4 matrix
*/
void matrix_transform(NCCAPixmap& img, matrix mat)
{
NCCAPixel p;
	for(int i = 0; i < img_w; ++i)
	{
		for(int j = 0; j < img_h; ++j)
		{
			p = getPixel(img,i,j);
			pix_mat(p,mat);
			setPixelColour(img,i,j,p);
		}
	}
}

/*
**void log_adjust_pixel(NCCAPixel& p)
**log based method for adjusting colour balance
*/
void log_adjust_pixel(NCCAPixel& p)
{
float sum = p.r + p.g + p.b;

double r,g,b;
double x = 1000;
r = x / pow(10 , 1/(p.r+0.00001) );
g = x / pow(10 , 1/(p.g+0.00001) );
b = x / pow(10 , 1/(p.b+0.00001) );

double scale = (p.r+p.g+p.b) / (r+g+b);

p.r = r*scale;
p.g = g*scale;
p.b = b*scale;
}

/*
**void log_adjust(NCCAPixmap& img)
**log based method for adjusting colour balance
*/
void log_adjust(NCCAPixmap& img)
{
NCCAPixel p;
	for(int i = 0; i < img_w; ++i)
	{
		for(int j = 0; j < img_h; ++j)
		{
			p = getPixel(img,i,j);
			log_adjust_pixel(p);
			setPixelColour(img,i,j,p);
		}
	}
}

