#include <iostream>
#include "spectral.h"
#include "pixels.h"

using namespace std;

extern float limits[6];
extern int img_w;
extern int img_h;

/*
**void xyz_spectral(NCCAPixmap& img, curvy& xyz, curvy** specimg)
**approximate spectral data from pixel values
*/
void xyz_spectral(NCCAPixmap& img, curvy& xyz, curvy** specimg)
{
xyz.fit_y(1,0);
NCCAPixel p;
int samples = xyz.sz(0);
double hold = 0.0f;
int ch = 0;
	for(int i = 0; i < img_w; ++i)//for each column
	{
		for(int j = 0; j < img_h; ++j)//for each row
		{
		p = getPixel(img,i,j);
			/*create an instance of the x,y,z function curves for this pixel*/
			specimg[i][j] = xyz;
			
			/*scale each function by its corresponding pixel*/
			specimg[i][j].scale(0, 1, p.r);
			specimg[i][j].scale(1, 1, p.g);
			specimg[i][j].scale(2, 1, p.b);
			
			/*sum the scaled channels together*/
			specimg[i][j].sum_channels();
		}
	}
}

/*
**void spectral_xyz(curvy** specimg, curvy& xyz, NCCAPixmap& img)
**copies the xyz functions to each pixel and scales each by corresponding channel at that pixel
*/
void spectral_xyz(curvy** specimg, curvy& xyz, NCCAPixmap& img)
{
NCCAPixel p;
double pixel[3];
double sum = 0.0f;
double sumr=0.0f,sumg=0.0f,sumb=0.0f;
int div = 0,id;
int samples = xyz.sz(0);double val = 10;
	for(int i = 0; i < img_w; ++i)
	{
		for(int j = 0; j < img_h; ++j)
		{
			for(int ch = 0; ch < 3; ch++)
				specimg[i][j].mult_ch(xyz,ch,ch);
		p.r = 1 * specimg[i][j].sum_y(0) / xyz.sum_y(0);
		p.g = 1 * specimg[i][j].sum_y(1) / xyz.sum_y(0);
		p.b = 1 * specimg[i][j].sum_y(2) / xyz.sum_y(0);
		
		setPixelColour(img,i,j,p);
		//print_pixel(p);
		}
	}
}

void filter(curvy** specimg, curvy& filter)
{
int samples = specimg[0][0].sz(0);
int offset = 0,c = 0,k = 0;
double hold = 0.0f;
double mult = 0.0f;
float step = 0.0f;
int ch = 0;
curvy temp;
	for(int i = 0; i < img_w; ++i)
	{
		for(int j = 0; j < img_h; ++j)
		{
			specimg[i][j].mult_ch(filter,1,0);//filter green
			
			specimg[i][j].mult_ch(filter,0,1);//filter red through magenta
			specimg[i][j].mult_ch(filter,2,1);//filter blue through magenta
			
			specimg[i][j].mult_ch(filter,0,2);//filter red through yellow
		
		/*scale curves to compensate for light reduction*/
		for(int ch=0;ch<3;++ch)
			specimg[i][j].scale(ch,1,3);
		}
	}
}

/*
**void filmss(curvy** specimg, curvy& filter, NCCAPixmap& img)
**calculate the effect of spectral sensitivity of film for each pixel
*/
void filmss(curvy** specimg, curvy& filter, NCCAPixmap& img)
{
	for(int i = 0; i < img_w; ++i)
	{
		for(int j = 0; j < img_h; ++j)
		{
		NCCAPixel p;
		
		/*multiply each channel by corresponding film sensitivity*/
		for(int ch = 0; ch < 3; ++ch)
			specimg[i][j].mult_ch(filter,ch,ch);
		
		/*set channels of pixel to exposure of curves and scale to account for multiplication by sensitivity*/
		p.r = specimg[i][j].sum_y(0)/5.6836;
		p.g = specimg[i][j].sum_y(1)/5.6856;
		p.b = specimg[i][j].sum_y(2)/5.02506;
		setPixelColour(img,i,j,p);
		}
	}
}

/*
**void dye_filter(NCCAPixmap& img, curvy& dyes,  curvy** specimg)
**simulate effect of dye curves on 'exposure' values
*/
void dye_filter(NCCAPixmap& img, curvy& dyes,  curvy** specimg)
{
/*copies of dye curves*/
curvy temp = dyes;
curvy temp1 = dyes;
int size = dyes.sz(0);
double hold = 0.0f;
NCCAPixel p;

/*calculate combined effect of complimentary dye pairs*/
for(int id = 0; id < size; ++id)
{
	for(int ch = 0; ch < 3; ++ch)
	{
		hold =  ( temp.get( (ch+1)%3 ,1,id) + temp.get( (ch+2)%3 ,1,id) );
		dyes.sety(ch,id,hold);
	}
}

/*vertically flip complimentary dye pair curves*/
for(int ch = 0; ch < 3; ++ch)
	dyes.flip_y(ch);
/*fit curves for complimentary dye pairs between range 0-1 */
dyes.fit_y(1.0,0.0);

/* fit copy of dye curves between 0 and 1 */
temp.fit_y(1.0,0.0);
if(MAKE_PS)temp.fprint("out_curves/ps1/temp_fit");

/*combine dye channels with their complimentary dye pairs*/
for(int ch = 0; ch < 3; ++ch)
{
	for(int id = 0; id < size; ++id)
	{
		hold = ( temp.get(ch,1,id) + dyes.get(ch,1,id) ) / 2;
		temp1.sety(ch,id,hold);
	}
}
/*reset dyes curves*/
dyes = temp1;

	for(int i = 0; i < img_w; ++i)
	{
		for(int j = 0; j < img_h; ++j)
		{
		p = getPixel(img,i,j);
		
		/*create instance of dye curves at this pixel*/
		specimg[i][j] = dyes;
		
		/*scale dye channels by pixel values that store 'exposure' from film filter function*/
		specimg[i][j].scale(0,1,p.r);
		specimg[i][j].scale(1,1,p.r);
		specimg[i][j].scale(2,1,p.r);
		
		/*sum channels to get spectral power distribution function*/
		specimg[i][j].sum_channels();
		}
	}
}

/*
**void fill_film_col_matrix(curvy& col_ss, matrix& mat)
**create rough colour transformation matrix from spectral curve data
*/
void fill_film_col_matrix(curvy& col_ss, matrix& mat)
{
int max[3] = {col_ss.max_y(0), col_ss.max_y(1), col_ss.max_y(2)};
double val = 0.0f;
	for(int i = 0; i < 3; ++i)
	{
		for(int j = 0; j < 3; ++j)
		{
			val = col_ss.get(j,1,max[i]);
			mat.set(i,j,val);
		}
	}
}







